@@ -64,19 +64,21 @@ static gboolean xa_process_stdout (GIOChannel *ioc, GIOCondition cond, XArchive
 
 			while (gtk_events_pending())
 				gtk_main_iteration();
+
+			return TRUE;
 		}
-	}
-	else
-	{
-		g_io_channel_shutdown(ioc, FALSE, NULL);
-		g_io_channel_unref(ioc);
-
-		xa_child_processed(XA_CHILD_STDOUT, cond == G_IO_HUP, archive);
-
-		return FALSE;
+		else if (status == G_IO_STATUS_AGAIN)
+			return TRUE;
+		else
+			cond &= ~G_IO_IN;
 	}
 
-	return TRUE;
+	g_io_channel_shutdown(ioc, FALSE, NULL);
+	g_io_channel_unref(ioc);
+
+	xa_child_processed(XA_CHILD_STDOUT, cond == G_IO_HUP, archive);
+
+	return FALSE;
 }
 
 static gboolean xa_process_stderr (GIOChannel *ioc, GIOCondition cond, XArchive *archive)
@@ -107,19 +109,21 @@ static gboolean xa_process_stderr (GIOChannel *ioc, GIOCondition cond, XArchive
 			}
 
 			g_free(line);
+
+			return TRUE;
 		}
-	}
-	else
-	{
-		g_io_channel_shutdown(ioc, FALSE, NULL);
-		g_io_channel_unref(ioc);
-
-		xa_child_processed(XA_CHILD_STDERR, cond == G_IO_HUP, archive);
-
-		return FALSE;
+		else if (status == G_IO_STATUS_AGAIN)
+			return TRUE;
+		else
+			cond &= ~G_IO_IN;
 	}
 
-	return TRUE;
+	g_io_channel_shutdown(ioc, FALSE, NULL);
+	g_io_channel_unref(ioc);
+
+	xa_child_processed(XA_CHILD_STDERR, cond == G_IO_HUP, archive);
+
+	return FALSE;
 }
 
 static void xa_process_exit (GPid pid, gint status, XArchive *archive)
@@ -227,7 +231,7 @@ static gpointer *xa_fill_archive_entry_columns_for_each_row (XArchive *archive,
 	return entry->columns;
 }
 
-static void xa_browse_dir_sidebar (XEntry *entry, GtkTreeStore *model, gchar *path, GtkTreeIter *containing_iter)
+static void xa_build_dir_sidebar (XEntry *entry, GtkTreeStore *model, gchar *path, GtkTreeIter *containing_iter)
 {
 	GtkTreeIter child_iter;
 
@@ -235,7 +239,7 @@ static void xa_browse_dir_sidebar (XEntry *entry, GtkTreeStore *model, gchar *pa
 		return;
 
 	if (strlen(entry->filename) == 0)
-		return xa_browse_dir_sidebar(entry->child, model, path, containing_iter);
+		return xa_build_dir_sidebar(entry->child, model, path, containing_iter);
 
 	if (entry->is_dir)
 	{
@@ -250,8 +254,8 @@ static void xa_browse_dir_sidebar (XEntry *entry, GtkTreeStore *model, gchar *pa
 
 		gtk_tree_store_set(model,&child_iter,0,"gtk-directory",1,entry->filename,2,entry,-1);
 	}
-	xa_browse_dir_sidebar(entry->child,model,NULL,&child_iter);
-	xa_browse_dir_sidebar(entry->next, model,NULL,containing_iter);
+	xa_build_dir_sidebar(entry->child, model, NULL, &child_iter);
+	xa_build_dir_sidebar(entry->next, model, NULL, containing_iter);
 
 }
 
@@ -387,7 +391,7 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 		g_error_free(error);
 
 		if (xa_main_window)
-			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
+			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->sorted, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
 
 		archive->status = XARCHIVESTATUS_ERROR;
 		return;
@@ -586,6 +590,23 @@ gboolean xa_run_command (XArchive *archive, const gchar *command)
 	return result;
 }
 
+gboolean xa_has_containing_directory (XArchive *archive)
+{
+	XEntry *entry;
+	guint n = 0;
+
+	entry = archive->root_entry->child;
+
+	while (entry)
+	{
+		if (++n > 1) break;
+
+		entry = entry->next;
+	}
+
+	return (n == 1) && archive->root_entry->child->is_dir;
+}
+
 gint xa_find_archive_index (gint page_num)
 {
 	GtkWidget *page;
@@ -780,17 +801,7 @@ void xa_fill_dir_sidebar(XArchive *archive,gboolean force_reload)
 		return;
 
 	gtk_tree_store_clear(GTK_TREE_STORE(archive_dir_model));
-	xa_browse_dir_sidebar(archive->root_entry,archive_dir_model,NULL,NULL);
-
-	if (force_reload)
-	{
-		GtkTreePath *path;
-
-		gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview)));
-		path = gtk_tree_path_new_first();
-		gtk_tree_view_expand_to_path(GTK_TREE_VIEW(archive_dir_treeview), path);
-		gtk_tree_path_free(path);
-	}
+	xa_build_dir_sidebar(archive->root_entry, archive_dir_model, NULL, NULL);
 }
 
 void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer user_data)
