@@ -554,21 +554,9 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 			if (gtk_widget_is_sensitive(dialog_data->ensure_directory) &&
 			    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->ensure_directory)))
 			{
-				XEntry *entry;
-				guint i = 0;
-
-				entry = archive->root_entry->child;
-
-				while (entry)
-				{
-					if (++i > 1) break;
-
-					entry = entry->next;
-				}
-
 				if (!gtk_widget_is_sensitive(dialog_data->extract_full) ||
 				    !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->extract_full)) ||
-				    (i != 1) || !archive->root_entry->child->is_dir)
+				    !xa_has_containing_directory(archive))
 				{
 					gchar *extraction_dir;
 
@@ -618,7 +606,7 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 				gtk_tree_selection_selected_foreach(selection,(GtkTreeSelectionForeachFunc)xa_concat_selected_filenames,&names);
 			if (xa_main_window)
 			{
-				xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
+				xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 				gtk_label_set_text(GTK_LABEL(total_label),_("Extracting files from archive, please wait..."));
 			}
 
@@ -679,7 +667,7 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(dialog_data->files_treeview),TRUE);
 	gtk_container_add (GTK_CONTAINER (scrolledwindow1),dialog_data->files_treeview);
 
-	gtk_drag_dest_set (dialog_data->files_treeview,GTK_DEST_DEFAULT_ALL,drop_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+	gtk_drag_dest_set(dialog_data->files_treeview, GTK_DEST_DEFAULT_ALL, drop_targets, 1, GDK_ACTION_COPY);
 	g_signal_connect (G_OBJECT (dialog_data->files_treeview),"drag-data-received",G_CALLBACK (xa_multi_extract_dialog_drag_data_received),dialog_data);
 
 	hbox2 = gtk_hbox_new (TRUE,5);
