@@ -329,7 +329,7 @@ static gboolean xa_dir_sidebar_drag_motion (GtkWidget *widget, GdkDragContext *c
 	}
 	/* This to set the focus on the dropped row */
 	gtk_tree_view_set_drag_dest_row(GTK_TREE_VIEW(widget),path,GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
-	gdk_drag_status(context, gdk_drag_context_get_suggested_action(context), time);
+	gdk_drag_status(context, GDK_ACTION_COPY, time);
 	return TRUE;
 }
 
@@ -397,20 +397,20 @@ static void xa_page_has_changed (GtkNotebook *notebook, GTK_COMPAT_SWITCH_PAGE_T
 		else
 			gtk_entry_set_text(GTK_ENTRY(location_entry),"\0");
 
-		if (GTK_IS_TREE_VIEW(archive[id]->treeview))
-			gtk_widget_grab_focus (GTK_WIDGET(archive[id]->treeview));
-
 		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview));
 		g_signal_handler_block(selection, selchghid);
 
 		xa_fill_dir_sidebar(archive[id],TRUE);
+		gtk_widget_grab_focus(archive_dir_treeview);
 
-		if (archive[id]->location_path)
+		if (archive[id]->location_path && *archive[id]->location_path)
 			xa_dir_sidebar_select_row(xa_find_entry_from_dirpath(archive[id], archive[id]->location_path));
+		else
+			gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview)));
 
 		g_signal_handler_unblock(selection, selchghid);
 	}
-	xa_set_button_state(1, 1, 1, 1, archive[id]->can_test, 1, archive[id]->can_add, archive[id]->can_extract, archive[id]->can_sfx, archive[id]->has_comment, archive[id]->output, archive[id]->has_password);
+	xa_set_button_state(1, 1, 1, 1, archive[id]->can_test, 1, archive[id]->can_add, archive[id]->can_extract, archive[id]->sorted, archive[id]->can_sfx, archive[id]->has_comment, archive[id]->output, archive[id]->has_password);
 }
 
 static void xa_select_by_pattern_dialog (GtkMenuItem *menuitem, gpointer user_data)
@@ -532,11 +532,10 @@ static void xa_handle_navigation_buttons (GtkToolButton *button, gpointer data)
 			}
 			/* Let's unselect the row in the dir_sidebar */
 			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (archive_dir_treeview));
-			if (selection != NULL)
-			{
-				gtk_tree_selection_get_selected (selection,&model,&iter);
+
+			if (gtk_tree_selection_get_selected(selection, &model, &iter))
 				gtk_tree_selection_unselect_iter(selection,&iter);
-			}
+
 			xa_update_window_with_archive_entries(archive[idx],NULL);
 			xa_restore_navigation(idx);
 		break;
@@ -561,11 +560,10 @@ static void xa_handle_navigation_buttons (GtkToolButton *button, gpointer data)
 
 			/* Let's unselect the row in the dir_sidebar */
 			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (archive_dir_treeview));
-			if (selection != NULL)
-			{
-				gtk_tree_selection_get_selected (selection,&model,&iter);
+
+			if (gtk_tree_selection_get_selected(selection, &model, &iter))
 				gtk_tree_selection_unselect_iter(selection,&iter);
-			}
+
 			new_entry = xa_find_entry_from_dirpath(archive[idx], archive[idx]->location_path);
 			xa_update_window_with_archive_entries(archive[idx],new_entry->prev);
 			xa_dir_sidebar_select_row(new_entry->prev);
@@ -1044,7 +1042,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_container_add (GTK_CONTAINER (scrolledwindow2),archive_dir_treeview);
 	gtk_widget_show(archive_dir_treeview);
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(archive_dir_model),1,GTK_SORT_ASCENDING);
-	gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(archive_dir_treeview),drop_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+	gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(archive_dir_treeview), drop_targets, 1, GDK_ACTION_COPY);
 	g_signal_connect (G_OBJECT (archive_dir_treeview),"row-collapsed",G_CALLBACK(xa_dir_sidebar_row_expanded),archive_dir_model);
 	g_signal_connect(G_OBJECT(archive_dir_treeview), "row-expanded", G_CALLBACK(xa_dir_sidebar_row_expanded), archive_dir_model);
 	g_signal_connect(G_OBJECT(archive_dir_treeview), "drag-data-received", G_CALLBACK(xa_dir_sidebar_drag_data_received), NULL);
@@ -1072,7 +1070,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_widget_show (GTK_WIDGET(notebook));
 	g_signal_connect(notebook, "switch-page", G_CALLBACK(xa_page_has_changed), NULL);
 
-	gtk_drag_dest_set (GTK_WIDGET(notebook),GTK_DEST_DEFAULT_ALL,drop_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+	gtk_drag_dest_set(GTK_WIDGET(notebook), GTK_DEST_DEFAULT_ALL, drop_targets, 1, GDK_ACTION_COPY);
 	g_signal_connect (G_OBJECT (notebook),"drag-data-received",G_CALLBACK (on_drag_data_received),NULL);
 
   	hbox_sb = gtk_hbox_new (FALSE,2);
@@ -1177,7 +1175,6 @@ void xa_add_page (XArchive *archive)
 	GtkWidget	*page_hbox,*label,*tab_label,*close_button,*image,*align;
 	gchar *filename_only, *label_utf8;
 	GtkRcStyle *rcstyle;
-	GtkRequisition size;
 
 	if (gtk_notebook_get_current_page(notebook) > -1)
 		gtk_notebook_set_show_tabs (notebook,TRUE);
@@ -1188,26 +1185,22 @@ void xa_add_page (XArchive *archive)
 	g_object_set(G_OBJECT(archive->page), "hscrollbar-policy", GTK_POLICY_AUTOMATIC, "vscrollbar-policy", GTK_POLICY_AUTOMATIC, NULL);
 	gtk_widget_show(archive->page);
 
-	page_hbox = gtk_hbox_new(FALSE,0);
+	page_hbox = gtk_hbox_new(FALSE, 4);
+	label = gtk_label_new(NULL);
 
 	filename_only = g_strrstr(archive->path[0], "/");
 	if (filename_only != NULL)
 	{
 		filename_only++;
 		label_utf8 = g_filename_display_name(filename_only);
-		label = gtk_label_new(label_utf8);
-		tab_label = gtk_label_new(label_utf8);
 	}
 	else
-	{
 		label_utf8 = g_filename_display_name(archive->path[0]);
-		label = gtk_label_new(label_utf8);
-		tab_label = gtk_label_new(label_utf8);
-	}
+
+	gtk_label_set_text(GTK_LABEL(label), xa_set_max_width_chars_ellipsize(label_utf8, 50, PANGO_ELLIPSIZE_START));
+	tab_label = gtk_label_new(label_utf8);
 	g_free(label_utf8);
 
-	gtk_label_set_max_width_chars(GTK_LABEL(label),50);
-	gtk_label_set_ellipsize(GTK_LABEL(label),PANGO_ELLIPSIZE_START);
 	gtk_box_pack_start(GTK_BOX(page_hbox),label,FALSE,FALSE,0);
 
 	close_button = gtk_button_new();
@@ -1221,11 +1214,9 @@ void xa_add_page (XArchive *archive)
 	gtk_widget_modify_style(close_button,rcstyle);
 	g_object_unref(rcstyle);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,GTK_ICON_SIZE_MENU);
-	gtk_widget_size_request(image,&size);
-	gtk_widget_set_size_request(close_button,size.width,size.height);
+	image = xa_main_window_find_image("xarchiver-close.png", 8);
 	gtk_container_add (GTK_CONTAINER(close_button),image);
-	align = gtk_alignment_new(1.0,0.0,0.0,0.0);
+	align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
 	gtk_container_add(GTK_CONTAINER(align),close_button);
 	gtk_box_pack_start(GTK_BOX(page_hbox),align,TRUE,TRUE,0);
 	gtk_widget_show_all(page_hbox);
@@ -1244,11 +1235,10 @@ void xa_add_page (XArchive *archive)
 	gtk_tree_selection_set_mode(sel,GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rubber_banding(GTK_TREE_VIEW(archive->treeview),TRUE);
 
-	gtk_drag_source_set (archive->treeview,GDK_BUTTON1_MASK,drag_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
+	gtk_drag_source_set(archive->treeview, GDK_BUTTON1_MASK, drag_targets, 1, GDK_ACTION_COPY);
 	g_signal_connect(sel, "changed", G_CALLBACK(xa_row_selected), archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"drag-begin",	G_CALLBACK (drag_begin),archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"drag-data-get", G_CALLBACK (drag_data_get),archive);
-	g_signal_connect (G_OBJECT (archive->treeview),"drag-end",		G_CALLBACK (drag_end),NULL);
 	g_signal_connect (G_OBJECT (archive->treeview),"row-activated", G_CALLBACK (xa_treeview_row_activated),archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"button-press-event",G_CALLBACK (xa_mouse_button_event),archive);
 }
@@ -1545,7 +1535,7 @@ GtkWidget *xa_create_archive_properties_window()
 	return archive_properties_window;
 }
 
-void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean save, gboolean test, gboolean close, gboolean add, gboolean extract, gboolean sfx, gboolean comment, GSList *output, gboolean password)
+void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean save, gboolean test, gboolean close, gboolean add, gboolean extract, gboolean sorted, gboolean sfx, gboolean comment, GSList *output, gboolean password)
 {
 	gtk_widget_set_sensitive(new1, new);
 	gtk_widget_set_sensitive(New_button, new);
@@ -1560,7 +1550,7 @@ void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean s
 	gtk_widget_set_sensitive(AddFile_button, add);
 	gtk_widget_set_sensitive(extract_menu, extract);
 	gtk_widget_set_sensitive(Extract_button, extract);
-	gtk_widget_set_sensitive(unsort_menu, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_sort_filename_column)) && close);
+	gtk_widget_set_sensitive(unsort_menu, sorted && close);
 	gtk_widget_set_sensitive(exe_menu, sfx);
 	gtk_widget_set_sensitive(comment_menu, comment);
 	gtk_widget_set_sensitive(select_all, close);
@@ -1633,14 +1623,13 @@ void xa_show_progress_bar (XArchive *archive)
 		gtk_misc_set_alignment (GTK_MISC (message),0,0.5);
 	}
 
-	progress->label = gtk_label_new("");
-	gtk_label_set_ellipsize(GTK_LABEL(progress->label), PANGO_ELLIPSIZE_END);
+	progress->label = gtk_label_new(NULL);
 	gtk_misc_set_alignment(GTK_MISC(progress->label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox2), progress->label, FALSE, FALSE, 12);
 	if (archive)
 	{
 		text = g_filename_display_name(archive->path[0]);
-		gtk_label_set_text(GTK_LABEL(progress->label), text);
+		gtk_label_set_text(GTK_LABEL(progress->label), xa_set_max_width_chars_ellipsize(text, 50, PANGO_ELLIPSIZE_END));
 		g_free(text);
 	}
 	else
@@ -1679,7 +1668,7 @@ void xa_increase_progress_bar (Progress *progress, gchar *filename, double perce
 		g_free(message);
 
 		basename = g_path_get_basename(filename);
-		markup = g_markup_printf_escaped("<b>%s</b>", basename);
+		markup = g_markup_printf_escaped("<b>%s</b>", xa_set_max_width_chars_ellipsize(basename, 50, PANGO_ELLIPSIZE_END));
 		g_free(basename);
 		gtk_label_set_markup(GTK_LABEL(progress->label), markup);
 		g_free (markup);
