@@ -804,6 +804,11 @@ static void xa_check_available_archivers ()
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.zip");
+		/* Android application package */
+		archiver[type].type = g_slist_append(archiver[type].type, "apk");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.apk");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('a'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* electronic publication */
 		archiver[type].type = g_slist_append(archiver[type].type, "epub");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.epub");
@@ -1047,22 +1052,10 @@ int main (int argc, char **argv)
 
 			if (opt_ensure_dir)
 			{
-				XEntry *entry;
-
 				current_dir = g_get_current_dir();
 				archive->extraction_dir = xa_escape_bad_chars(current_dir, ESCAPES);
 
-				i = 0;
-				entry = archive->root_entry->child;
-
-				while (entry)
-				{
-					if (++i > 1) break;
-
-					entry = entry->next;
-				}
-
-				if ((i != 1) || !archive->root_entry->child->is_dir)
+				if (!xa_has_containing_directory(archive))
 				{
 					extraction_dir = xa_create_containing_directory(archive, current_dir);
 
