@@ -16,6 +16,7 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <dirent.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include "open-with-dlg.h"
@@ -29,6 +30,7 @@ typedef struct
 {
 	gchar *name;
 	gchar *exec;
+	gboolean multiple;
 	GdkPixbuf *icon;
 } App;
 
@@ -161,7 +163,7 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 	gchar *app_name = NULL, *app_exec = NULL, *app_icon = NULL;
 	GIOStatus status;
 	GIOChannel *file;
-	gboolean has_mimetype = FALSE;
+	gboolean group_line = FALSE, app_multiple = FALSE, has_mimetype = FALSE;
 	const gchar * const *langs, * const *l;
 	gint size;
 
@@ -177,6 +179,15 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 		status = g_io_channel_read_line (file, &line, NULL, NULL, NULL);
 		if (line != NULL)
 		{
+			if (strcmp(g_strchomp(line), "[Desktop Entry]") == 0)
+			{
+				group_line = TRUE;
+				continue;
+			}
+			if (group_line && *line == '[')
+				break;
+			if (!group_line)
+				continue;
 			if (g_str_has_prefix(line, "Name["))
 			{
 				l = langs;
@@ -188,7 +199,7 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 					if (g_str_has_prefix(line, key))
 					{
 						g_free(app_name);
-						app_name = g_strndup(line + strlen(key), strlen(line) - strlen(key) - 1);
+						app_name = g_strndup(line + strlen(key), strlen(line) - strlen(key));
 						g_free(key);
 						break;
 					}
@@ -199,16 +210,19 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 			}
 			if (!app_name && g_str_has_prefix(line, "Name="))
 			{
-				app_name = g_strndup(line + 5,(strlen(line)-6));
+				app_name = g_strndup(line + 5, strlen(line) - 5);
 				continue;
 			}
 			if (g_str_has_prefix(line,"Exec="))
 			{
 				app_exec = strstr(line, " %");
 				if (app_exec)
+				{
+					app_multiple = (app_exec[2] == 'F' || app_exec[2] == 'U');
 					app_exec = g_strndup(line + 5, app_exec - (line + 5));
+				}
 				else
-					app_exec = g_strndup(line + 5, strlen(line) - 6);
+					app_exec = g_strndup(line + 5, strlen(line) - 5);
 				continue;
 			}
 			if (g_str_has_prefix(line,"Icon="))
@@ -217,7 +231,7 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 				if (app_icon)
 					app_icon = g_strndup(line + 5,app_icon - (line+5));
 				else
-					app_icon = g_strndup(line + 5,(strlen(line)-6));
+					app_icon = g_strndup(line + 5, strlen(line) - 5);
 				continue;
 			}
 			if (g_str_has_prefix(line,"MimeType="))
@@ -249,11 +263,27 @@ static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_wi
 
 			app->name = app_name;
 			app->exec = app_exec;
+			app->multiple = app_multiple;
 
-			if (gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size)) == 0)
-				size = 40;
-			else
-				size = 24;
+			switch (gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size)))
+			{
+				case 0:
+					size = 24;
+					break;
+
+				case 1:
+				case 2:
+					size = 32;
+					break;
+
+				case 3:
+					size = 40;
+					break;
+
+				default:
+					size = 48;
+					break;
+			}
 
 			if (app_icon)
 			{
@@ -337,7 +367,7 @@ void xa_create_open_with_dialog (const gchar *filename, gchar *filenames, gint n
 	gtk_window_set_type_hint (GTK_WINDOW (data->dialog1), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_transient_for(GTK_WINDOW(data->dialog1),GTK_WINDOW(xa_main_window));
 	gtk_container_set_border_width (GTK_CONTAINER (data->dialog1),5);
-	gtk_widget_set_size_request(data->dialog1,380,380);
+	gtk_widget_set_size_request(data->dialog1, 380, 425);
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(data->dialog1));
 
 	vbox1 = gtk_vbox_new (FALSE, 5);
@@ -446,14 +476,22 @@ void xa_create_open_with_dialog (const gchar *filename, gchar *filenames, gint n
 	{
 		App *app = apps->data;
 
-		gtk_list_store_append(apps_liststore, &iter);
-		gtk_list_store_set(apps_liststore, &iter, 0, app->icon, 1, app->name, 2, app->exec, -1);
+		if (app->multiple || (nr == 1))
+		{
+			gtk_list_store_append(apps_liststore, &iter);
+			gtk_list_store_set(apps_liststore, &iter, 0, app->icon, 1, app->name, 2, app->exec, -1);
+		}
 
 		apps = apps->next;
 	}
 
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(apps_liststore), &iter);
-	gtk_tree_selection_select_iter(gtk_tree_view_get_selection (GTK_TREE_VIEW (apps_treeview)),&iter);
+	if (data->apps)
+	{
+		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(apps_liststore), &iter);
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(apps_treeview)), &iter);
+	}
+	else
+		gtk_widget_set_sensitive(okbutton1, FALSE);
 
 	g_signal_connect (G_OBJECT (apps_treeview),	"row-activated",G_CALLBACK(xa_open_with_dialog_row_selected),data);
 	g_signal_connect(G_OBJECT(apps_treeview), "button-release-event", G_CALLBACK(xa_open_with_dialog_mouse_button_event), data);
