@@ -72,7 +72,7 @@ static gchar *xa_rpm2cpio (XArchive *archive)
 		fclose (stream);
 		return g_strconcat(_("Can't fseek to position 104:"), " ", g_strerror(errno), NULL);
 	}
-	if (fread(bytes, 1, HDRSIG_ENTRY_INFO_LEN, stream) != HDRSIG_ENTRY_INFO_LEN)
+	if (fread(bytes, sizeof(*bytes), HDRSIG_ENTRY_INFO_LEN, stream) != HDRSIG_ENTRY_INFO_LEN)
 	{
 		fclose ( stream );
 		return g_strconcat(_("Can't read data from file:"), " ", g_strerror(errno), NULL);
@@ -88,7 +88,7 @@ static gchar *xa_rpm2cpio (XArchive *archive)
 		fclose (stream);
 		return g_strconcat(_("Can't fseek in file:"), " ", g_strerror(errno), NULL);
 	}
-	if (fread(bytes, 1, HDRSIG_ENTRY_INFO_LEN, stream) != HDRSIG_ENTRY_INFO_LEN)
+	if (fread(bytes, sizeof(*bytes), HDRSIG_ENTRY_INFO_LEN, stream) != HDRSIG_ENTRY_INFO_LEN)
 	{
 		fclose ( stream );
 		return g_strconcat(_("Can't read data from file:"), " ", g_strerror(errno), NULL);
@@ -293,16 +293,20 @@ gboolean xa_rpm_extract (XArchive *archive, GSList *file_list)
 	gboolean result;
 
 	files = xa_quote_filenames(file_list, "*?[]\"", FALSE);
+	archive->child_dir = g_strdup(archive->extraction_dir);
 	command = g_strconcat(archiver[archive->type].program[0], " -id",
 	                      archive->do_touch ? "" : " -m",
 	                      archive->do_overwrite ? " -u" : "",
 	                      " -I ", archive->working_dir, "/xa-tmp.cpio",
-	                      " -D ", archive->extraction_dir, files->str, NULL);
+	                      files->str, NULL);
 
 	g_string_free(files,TRUE);
 
 	result = xa_run_command(archive, command);
 	g_free(command);
 
+	g_free(archive->child_dir);
+	archive->child_dir = NULL;
+
 	return result;
 }
