@@ -23,7 +23,7 @@
 #include <gtk/gtk.h>
 #include "archive.h"
 
-#define ESCAPES " \"'\\"
+#define ESCAPES " \"#'\\"
 
 #ifndef HAVE_MKDTEMP
 gchar *mkdtemp(gchar *);
@@ -45,6 +45,7 @@ GString *xa_quote_filenames(GSList *, const gchar *, gboolean);
 gchar *xa_quote_shell_command(const gchar *, gboolean);
 void xa_recurse_local_directory(gchar *, GSList **, gboolean);
 gchar *xa_remove_level_from_path(const gchar *);
+gchar *xa_set_max_width_chars_ellipsize(const gchar *, gint, PangoEllipsizeMode);
 void xa_set_window_title(GtkWidget *, gchar *);
 GSList *xa_slist_copy(GSList *);
 
