@@ -17,7 +17,6 @@
  */
 
 #include "config.h"
-#include <endian.h>
 #include <errno.h>
 #include <stdint.h>
 #include <string.h>
@@ -39,6 +38,13 @@
 #include "support.h"
 #include "tar.h"
 
+#ifdef __APPLE__
+#include <libkern/OSByteOrder.h>
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+#else
+#include <endian.h>
+#endif
+
 #ifdef HAVE_SOCKET
 #include "socket.h"
 #endif
@@ -753,6 +759,22 @@ static void xa_clipboard_cut_copy_operation (XArchive *archive, XAClipboardMode
 	(*archive->archiver->extract) (archive,files);
 }
 
+static void xa_expand_containing_directory (XArchive *archive)
+{
+	if (xa_has_containing_directory(archive))
+	{
+		GtkTreePath *path;
+		GtkTreeSelection *selection;
+
+		path = gtk_tree_path_new_first();
+		gtk_tree_view_expand_to_path(GTK_TREE_VIEW(archive_dir_treeview), path);
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview));
+		gtk_tree_selection_select_path(selection, path);
+		gtk_tree_selection_unselect_path(selection, path);
+		gtk_tree_path_free(path);
+	}
+}
+
 void xa_child_processed (XAChildProcess process, gboolean success, XArchive *archive)
 {
 	static gboolean okay[XA_CHILD_PROCS];
@@ -766,7 +788,7 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 		if (xa_main_window)
 		{
 			gtk_widget_set_sensitive(Stop_button, FALSE);
-			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
+			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->sorted, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
 		}
 	}
 
@@ -781,11 +803,11 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 			{
 				if (archive->parse_output)
 				{
+					gtk_widget_grab_focus(archive_dir_treeview);
 					xa_update_window_with_archive_entries(archive, NULL);
+					xa_expand_containing_directory(archive);
 					gtk_tree_view_set_model(GTK_TREE_VIEW(archive->treeview), archive->model);
 					g_object_unref(archive->model);
-
-					gtk_widget_grab_focus(archive->treeview);
 				}
 
 				xa_set_statusbar_message_for_displayed_rows(archive);
@@ -830,12 +852,9 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 void xa_reload_archive_content (XArchive *this_archive)
 {
 	XEntry *entry;
-	gint idx;
 
 	//TODO: have the status bar notyfing the reload
 
-	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
-
 	g_slist_free(this_archive->forward);
 	this_archive->forward = NULL;
 
@@ -854,9 +873,6 @@ void xa_reload_archive_content (XArchive *this_archive)
 	/* this reload will be called internally during adding and deleting */
 	this_archive->status = XARCHIVESTATUS_RELOAD;
 	(*this_archive->archiver->list)(this_archive);
-
-	if (strcmp(this_archive->path[0], archive[idx]->path[0]) == 0)
-		xa_fill_dir_sidebar(this_archive, TRUE);
 }
 
 void xa_show_archive_output (GtkMenuItem *menuitem, XArchive *this_archive)
@@ -945,7 +961,7 @@ void xa_new_archive (GtkMenuItem *menuitem,gpointer user_data)
 		return;
 
 	xa_add_page (archive[current_page]);
-	xa_set_button_state(1, 1, 1, 1, archive[current_page]->can_test, 1, archive[current_page]->can_add, archive[current_page]->can_extract, archive[current_page]->can_sfx, archive[current_page]->has_comment, archive[current_page]->output, archive[current_page]->has_password);
+	xa_set_button_state(1, 1, 1, 1, archive[current_page]->can_test, 1, archive[current_page]->can_add, archive[current_page]->can_extract, archive[current_page]->sorted, archive[current_page]->can_sfx, archive[current_page]->has_comment, archive[current_page]->output, archive[current_page]->has_password);
     xa_disable_delete_buttons(FALSE);
 
 	xa_set_window_title(xa_main_window, archive[current_page]->path[0]);
@@ -1082,13 +1098,11 @@ void xa_open_archive (GtkWidget *widget, gchar *path)
 	xa_disable_delete_buttons (FALSE);
 	g_free (path);
 
-	xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
+	xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 	gtk_label_set_text(GTK_LABEL(total_label),_("Opening archive, please wait..."));
 
 	archive[current_page]->status = XARCHIVESTATUS_LIST;
 	(*archive[current_page]->archiver->list)(archive[current_page]);
-
-	xa_fill_dir_sidebar(archive[current_page],TRUE);
 }
 
 void xa_test_archive (GtkMenuItem *menuitem,gpointer user_data)
@@ -1270,7 +1284,7 @@ void xa_close_archive (GtkWidget *widget, gpointer page)
 		gtk_widget_set_sensitive (home_button,FALSE);
 		gtk_widget_set_sensitive (deselect_all,FALSE);
 		xa_disable_delete_buttons (FALSE);
-		xa_set_button_state(1, 1, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
+		xa_set_button_state(1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 		xa_set_window_title (xa_main_window,NULL);
 		gtk_tree_store_clear(GTK_TREE_STORE(archive_dir_model));
 		gtk_entry_set_text(GTK_ENTRY(location_entry),"");
@@ -1620,13 +1634,13 @@ void xa_about (GtkMenuItem *menuitem,gpointer user_data)
     static GtkWidget *about = NULL;
     const char *authors[] =
     {
-        "",
-        "Main developer:",
-        "Giuseppe Torelli <colossus73@gmail.com>",
         "",
         "This version:",
         MAINTAINER " <ib@wupperonline.de>",
         "",
+        "Original developer:",
+        "Giuseppe Torelli <colossus73@gmail.com>",
+        "",
         "Archive navigation code:",
         "John Berthels",
         "",
@@ -1795,8 +1809,6 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	}
 	else if (memcmp(magic, "\xed\xab\xee\xdb", 4) == 0)
 		xa.type = XARCHIVETYPE_RPM;
-	else if (isTar(file))
-		xa.type = XARCHIVETYPE_TAR;
 	else if (memcmp(magic, "\xfd" "7zXZ" "\x00", 6) == 0)
 		xa.type = XARCHIVETYPE_XZ;
 	else if (memcmp(magic, "PK" "\x03\x04", 4) == 0 ||
@@ -1804,7 +1816,9 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	{
 		xa.type = XARCHIVETYPE_ZIP;
 
-		if (g_str_has_suffix(filename, ".epub"))
+		if (g_str_has_suffix(filename, ".apk"))
+			xa.tag = 'a';
+		else if (g_str_has_suffix(filename, ".epub"))
 			xa.tag = 'e';
 		else if (g_str_has_suffix(filename, ".jar"))
 			xa.tag = 'j';
@@ -1816,6 +1830,9 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	else if (memcmp(magic + 1, "\xb5\x2f\xfd", 3) == 0 &&
 	         (*magic == '\x1e' || (*magic >= '\x22' && *magic <= '\x28')))
 		xa.type = XARCHIVETYPE_ZSTD;
+	/* partly heuristic methods must come last */
+	else if (isTar(file))
+		xa.type = XARCHIVETYPE_TAR;
 
 	fclose(file);
 
@@ -1836,8 +1853,13 @@ void xa_create_liststore (XArchive *archive, const gchar *titles[])
 	gtk_tree_view_set_model ( GTK_TREE_VIEW (archive->treeview),GTK_TREE_MODEL (archive->liststore));
 
 	archive->model = GTK_TREE_MODEL(archive->liststore);
+
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_sort_filename_column)))
-		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(archive->model),1,GTK_SORT_ASCENDING);
+	{
+		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(archive->model), 1, GTK_SORT_ASCENDING);
+		archive->sorted = TRUE;
+	}
+
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(archive->liststore), 1, (GtkTreeIterCompareFunc) xa_sort_dirs_before_files, archive, NULL);
 
 	g_object_ref(archive->model);
@@ -1846,7 +1868,6 @@ void xa_create_liststore (XArchive *archive, const gchar *titles[])
 	/* icon and filename */
 	column = gtk_tree_view_column_new();
 	archive->pixbuf_renderer = gtk_cell_renderer_pixbuf_new();
-	g_object_set(G_OBJECT(archive->pixbuf_renderer), "stock-size", (3 - gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size))), NULL);
 	gtk_tree_view_column_pack_start(column, archive->pixbuf_renderer, FALSE);
 	gtk_tree_view_column_set_attributes(column, archive->pixbuf_renderer, "pixbuf", 0, NULL);
 
@@ -2076,15 +2097,19 @@ void xa_row_selected (GtkTreeSelection *selection,XArchive *archive)
 	g_free(msg);
 }
 
-void drag_begin (GtkWidget *treeview1,GdkDragContext *context,XArchive *archive)
+void drag_begin (GtkWidget *widget, GdkDragContext *context, XArchive *archive)
 {
     GtkTreeSelection *selection;
     GtkTreeIter       iter;
     GList            *row_list;
 	XEntry *entry;
 
-	gtk_drag_source_set_icon_name (archive->treeview,"xarchiver");
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (archive->treeview));
+	if (archive->child_pid)
+		gtk_drag_source_set_icon_stock(widget, "gtk-stop");
+	else
+		gtk_drag_source_set_icon_name(widget, "xarchiver");
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 
 	row_list = gtk_tree_selection_get_selected_rows (selection,NULL);
 	if (row_list == NULL)
@@ -2104,36 +2129,43 @@ void drag_begin (GtkWidget *treeview1,GdkDragContext *context,XArchive *archive)
 	g_list_free (row_list);
 }
 
-void drag_end (GtkWidget *widget, GdkDragContext *context, gpointer user_data)
-{
-	/* Nothing to do */
-}
-
-void drag_data_get (GtkWidget *widget,GdkDragContext *dc,GtkSelectionData *selection_data,guint info,guint t,XArchive *archive)
+void drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *data, guint info, guint time, XArchive *archive)
 {
 	GtkTreeSelection *selection;
-	GList *row_list = NULL;
+	GList *row_list;
 	GSList *names = NULL;
+	gint length;
 	guchar *_destination;
 	gchar *destination, *to_send, *extraction_dir;
 
+	if (archive->child_pid)
+		return;
+
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (archive->treeview));
 	row_list = gtk_tree_selection_get_selected_rows (selection,NULL);
 
 	if (row_list == NULL)
 		return;
 
-	if (archive->child_pid!= 0)
-	{
-		xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't perform another extraction:"),_("Please wait until the completion of the current one!"));
-		return;
-	}
-	gdk_property_get(gdk_drag_context_get_source_window(dc),
-						gdk_atom_intern ("XdndDirectSave0",FALSE),
-						gdk_atom_intern ("text/plain",FALSE),
-						0,4096,FALSE,NULL,NULL,NULL,&_destination );
+	g_list_foreach(row_list, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free(row_list);
+
+	gdk_property_get(gdk_drag_context_get_source_window(context),
+	                 gdk_atom_intern("XdndDirectSave0", FALSE),
+	                 gdk_atom_intern("text/plain", FALSE),
+	                 0, 4096, FALSE, NULL, NULL, &length, &_destination);
+
 	if (_destination)
 	{
+		_destination = g_realloc(_destination, length + 1);
+		_destination[length] = 0;
+
+		if (strcmp((const char *) _destination, XDS_FILENAME) == 0)
+		{
+			g_free(_destination);
+			return;
+		}
+
 		destination = g_filename_from_uri((gchar*)_destination,NULL,NULL);
 		g_free(_destination);
 
@@ -2146,7 +2178,7 @@ void drag_data_get (GtkWidget *widget,GdkDragContext *dc,GtkSelectionData *selec
 		{
 			if (!xa_check_password(archive))
 			{
-				gtk_drag_finish(dc, FALSE, FALSE, t);
+				gtk_drag_finish(context, FALSE, FALSE, time);
 				g_free(extraction_dir);
 				return;
 			}
@@ -2172,17 +2204,16 @@ void drag_data_get (GtkWidget *widget,GdkDragContext *dc,GtkSelectionData *selec
 			g_free(archive->extraction_dir);
 			archive->extraction_dir = xa_escape_bad_chars(extraction_dir, ESCAPES);
 
+			gtk_label_set_text(GTK_LABEL(total_label), _("Extracting files from archive, please wait..."));
 			archive->status = XARCHIVESTATUS_EXTRACT;
 			(*archive->archiver->extract) (archive,names);
 
-			g_list_foreach (row_list,(GFunc) gtk_tree_path_free,NULL);
-			g_list_free (row_list);
 			to_send = "S";
 		}
 
 		g_free(extraction_dir);
 
-		gtk_selection_data_set(selection_data, gtk_selection_data_get_target(selection_data), 8, (guchar *) to_send, 1);
+		gtk_selection_data_set(data, gtk_selection_data_get_target(data), 8, (const guchar *) to_send, 1);
 	}
 }
 
@@ -2523,18 +2554,9 @@ gboolean xa_mouse_button_event (GtkWidget *widget, GdkEventButton *event, XArchi
 			xa_treeview_row_activated(GTK_TREE_VIEW(archive->treeview), path, NULL, archive);
 		else if (event->button == 3)
 		{
-			if (selected > 1 || entry->is_dir)
-			{
-				gtk_widget_set_sensitive(open_popupmenu, FALSE);
-				gtk_widget_set_sensitive(view, FALSE);
-				gtk_widget_set_sensitive(rrename, FALSE);
-			}
-			else
-			{
-				gtk_widget_set_sensitive(open_popupmenu, archive->can_extract);
-				gtk_widget_set_sensitive(view, archive->can_extract);
-				gtk_widget_set_sensitive(rrename, can_rename(archive));
-			}
+			gtk_widget_set_sensitive(open_popupmenu, !entry->is_dir && archive->can_extract);
+			gtk_widget_set_sensitive(view, (selected == 1) && !entry->is_dir && archive->can_extract);
+			gtk_widget_set_sensitive(rrename, (selected == 1) && !entry->is_dir && can_rename(archive));
 
 			clipboard = gtk_clipboard_get(XA_CLIPBOARD);
 			clipboard_selection = gtk_clipboard_wait_for_contents(clipboard, XA_INFO_LIST);
@@ -2817,7 +2839,7 @@ void xa_treeview_row_activated(GtkTreeView *tree_view,GtkTreePath *path,GtkTreeV
 
 void xa_update_window_with_archive_entries (XArchive *archive,XEntry *entry)
 {
-	gboolean reload = FALSE;
+	gboolean reload;
 	GdkPixbuf *pixbuf = NULL;
 	GtkTreeIter iter;
 	guint i;
@@ -2831,32 +2853,24 @@ void xa_update_window_with_archive_entries (XArchive *archive,XEntry *entry)
 	else
 		archive->current_entry = entry;
 
+	g_free(archive->location_path);
+	archive->location_path = NULL;
+
 	if (entry == NULL)
 	{
+		reload = TRUE;
 		entry = archive->root_entry->child;
 		gtk_entry_set_text(GTK_ENTRY(location_entry),"\0");
-		reload = TRUE;
-		if (archive->location_path != NULL)
-		{
-			g_free(archive->location_path);
-			archive->location_path = NULL;
-		}
+		gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview)));
+
 		gtk_widget_set_sensitive(back_button,FALSE);
 		gtk_widget_set_sensitive(up_button,FALSE);
 		gtk_widget_set_sensitive(home_button,FALSE);
 	}
 	else
 	{
-		if (archive->location_path != NULL)
-		{
-			g_free(archive->location_path);
-			archive->location_path = NULL;
-		}
-		if (archive->back == NULL)
-			gtk_widget_set_sensitive(back_button,FALSE);
-		else
-			gtk_widget_set_sensitive(back_button,TRUE);
-
+		reload = FALSE;
+		gtk_widget_set_sensitive(back_button, archive->back != NULL);
 		gtk_widget_set_sensitive(up_button,TRUE);
 		gtk_widget_set_sensitive(home_button,TRUE);
 		archive->location_path = xa_build_full_path_name_from_entry(entry);
@@ -2884,10 +2898,29 @@ void xa_update_window_with_archive_entries (XArchive *archive,XEntry *entry)
 		else
 			filename = entry->filename;
 
-		if (gtk_combo_box_get_active (GTK_COMBO_BOX(prefs_window->combo_icon_size)) == 0)
-			size = 30;
-		else
-			size = 20;
+		switch (gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size)))
+		{
+			case 0:
+				size = 20;
+				break;
+
+			case 1:
+				size = 22;
+				break;
+
+			case 2:
+				size = 24;
+				break;
+
+			case 3:
+				size = 30;
+				break;
+
+			default:
+				size = 32;
+				break;
+		}
+
 		pixbuf = xa_get_pixbuf_icon_from_cache(filename,size);
 		gtk_list_store_set(archive->liststore, &iter, archive->columns - 1, entry, -1);
 		gtk_list_store_set (archive->liststore,&iter,0,pixbuf,1,entry->filename,-1);
@@ -2926,25 +2959,46 @@ void xa_show_multi_extract_dialog (GtkMenuItem *menu_item, gpointer user_data)
 void xa_unsort (GtkMenuItem *menu_item, gpointer user_data)
 {
 	gint idx;
+	XArchiveStatus status;
 
 	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
 	if (idx < 0)
 		return;
 
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(archive[idx]->model), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(archive[idx]->model), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, (GtkSortType) 0);
+	gtk_widget_set_sensitive(unsort_menu, FALSE);
+	archive[idx]->sorted = FALSE;
+
+	status = archive[idx]->status;
+	archive[idx]->status = XARCHIVESTATUS_RELOAD;
 	xa_update_window_with_archive_entries(archive[idx], NULL);
+	archive[idx]->status = status;
 }
 
 GtkWidget *xa_main_window_find_image (gchar *filename, GtkIconSize size)
 {
-  gchar *path;
-  GdkPixbuf *file_pixbuf;
+  gchar *name, *ext, *path;
+  GdkPixbuf *file_pixbuf = NULL;
   GtkWidget *file_image;
 
-  path = g_strconcat(PIXMAPSDIR, "/", filename, NULL);
-  file_pixbuf = gdk_pixbuf_new_from_file(path, NULL);
-  g_free(path);
+  name = g_strdup(filename);
+  ext = g_strrstr(name, ".png");
+
+  if (ext)
+  {
+    *ext = 0;
+    file_pixbuf = gtk_icon_theme_load_icon(icon_theme, name, size, 0, NULL);
+  }
+
+  g_free(name);
+
+  if (file_pixbuf == NULL)
+  {
+    path = g_strconcat(PIXMAPSDIR, "/", filename, NULL);
+    file_pixbuf = gdk_pixbuf_new_from_file(path, NULL);
+    g_free(path);
+  }
 
   if (file_pixbuf == NULL)
     file_image = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE, size);
